@@ -1,19 +1,21 @@
-import typing
-
+import logging
 import os
+from collections.abc import Sequence
+from typing import Union
 
+from mitmproxy.log import ALERT
 from mitmproxy.utils import emoji
 from mitmproxy import ctx, hooks
 from mitmproxy import exceptions
 from mitmproxy import command
 from mitmproxy import flow
 from mitmproxy import optmanager
-from mitmproxy import platform
-from mitmproxy.net import server_spec
 from mitmproxy.net.http import status_codes
 import mitmproxy.types
 
 
+logger = logging.getLogger(__name__)
+
 CONF_DIR = "~/.mitmproxy"
 LISTEN_PORT = 8080
 
@@ -25,22 +27,6 @@ class Core:
             raise exceptions.OptionsError(
                 "add_upstream_certs_to_client_chain requires the upstream_cert option to be enabled."
             )
-        if "mode" in updated:
-            mode = opts.mode
-            if mode.startswith("reverse:") or mode.startswith("upstream:"):
-                try:
-                    server_spec.parse_with_mode(mode)
-                except ValueError as e:
-                    raise exceptions.OptionsError(str(e)) from e
-            elif mode == "transparent":
-                if not platform.original_addr:
-                    raise exceptions.OptionsError(
-                        "Transparent mode not supported on this platform."
-                    )
-            elif mode not in ["regular", "socks5"]:
-                raise exceptions.OptionsError(
-                    "Invalid mode specification: %s" % mode
-                )
         if "client_certs" in updated:
             if opts.client_certs:
                 client_certs = os.path.expanduser(opts.client_certs)
@@ -52,10 +38,10 @@ class Core:
     @command.command("set")
     def set(self, option: str, *value: str) -> None:
         """
-            Set an option. When the value is omitted, booleans are set to true,
-            strings and integers are set to None (if permitted), and sequences
-            are emptied. Boolean values can be true, false or toggle.
-            Multiple values are concatenated with a single space.
+        Set an option. When the value is omitted, booleans are set to true,
+        strings and integers are set to None (if permitted), and sequences
+        are emptied. Boolean values can be true, false or toggle.
+        Multiple values are concatenated with a single space.
         """
         if value:
             specs = [f"{option}={v}" for v in value]
@@ -67,9 +53,9 @@ class Core:
             raise exceptions.CommandError(e) from e
 
     @command.command("flow.resume")
-    def resume(self, flows: typing.Sequence[flow.Flow]) -> None:
+    def resume(self, flows: Sequence[flow.Flow]) -> None:
         """
-            Resume flows if they are intercepted.
+        Resume flows if they are intercepted.
         """
         intercepted = [i for i in flows if i.intercepted]
         for f in intercepted:
@@ -78,9 +64,9 @@ class Core:
 
     # FIXME: this will become view.mark later
     @command.command("flow.mark")
-    def mark(self, flows: typing.Sequence[flow.Flow], marker: mitmproxy.types.Marker) -> None:
+    def mark(self, flows: Sequence[flow.Flow], marker: mitmproxy.types.Marker) -> None:
         """
-            Mark flows.
+        Mark flows.
         """
         updated = []
         if marker not in emoji.emoji:
@@ -93,9 +79,9 @@ class Core:
 
     # FIXME: this will become view.mark.toggle later
     @command.command("flow.mark.toggle")
-    def mark_toggle(self, flows: typing.Sequence[flow.Flow]) -> None:
+    def mark_toggle(self, flows: Sequence[flow.Flow]) -> None:
         """
-            Toggle mark for flows.
+        Toggle mark for flows.
         """
         for i in flows:
             if i.marked:
@@ -105,34 +91,34 @@ class Core:
         ctx.master.addons.trigger(hooks.UpdateHook(flows))
 
     @command.command("flow.kill")
-    def kill(self, flows: typing.Sequence[flow.Flow]) -> None:
+    def kill(self, flows: Sequence[flow.Flow]) -> None:
         """
-            Kill running flows.
+        Kill running flows.
         """
         updated = []
         for f in flows:
             if f.killable:
                 f.kill()
                 updated.append(f)
-        ctx.log.alert("Killed %s flows." % len(updated))
+        logger.log(ALERT, "Killed %s flows." % len(updated))
         ctx.master.addons.trigger(hooks.UpdateHook(updated))
 
     # FIXME: this will become view.revert later
     @command.command("flow.revert")
-    def revert(self, flows: typing.Sequence[flow.Flow]) -> None:
+    def revert(self, flows: Sequence[flow.Flow]) -> None:
         """
-            Revert flow changes.
+        Revert flow changes.
         """
         updated = []
         for f in flows:
             if f.modified():
                 f.revert()
                 updated.append(f)
-        ctx.log.alert("Reverted %s flows." % len(updated))
+        logger.log(ALERT, "Reverted %s flows." % len(updated))
         ctx.master.addons.trigger(hooks.UpdateHook(updated))
 
     @command.command("flow.set.options")
-    def flow_set_options(self) -> typing.Sequence[str]:
+    def flow_set_options(self) -> Sequence[str]:
         return [
             "host",
             "status_code",
@@ -144,16 +130,11 @@ class Core:
 
     @command.command("flow.set")
     @command.argument("attr", type=mitmproxy.types.Choice("flow.set.options"))
-    def flow_set(
-        self,
-        flows: typing.Sequence[flow.Flow],
-        attr: str,
-        value: str
-    ) -> None:
+    def flow_set(self, flows: Sequence[flow.Flow], attr: str, value: str) -> None:
         """
-            Quickly set a number of common values on flows.
+        Quickly set a number of common values on flows.
         """
-        val: typing.Union[int, str] = value
+        val: Union[int, str] = value
         if attr == "status_code":
             try:
                 val = int(val)  # type: ignore
@@ -178,7 +159,7 @@ class Core:
                         req.url = val
                     except ValueError as e:
                         raise exceptions.CommandError(
-                            "URL {} is invalid: {}".format(repr(val), e)
+                            f"URL {repr(val)} is invalid: {e}"
                         ) from e
                 else:
                     self.rupdate = False
@@ -199,12 +180,12 @@ class Core:
                 updated.append(f)
 
         ctx.master.addons.trigger(hooks.UpdateHook(updated))
-        ctx.log.alert("Set {} on  {} flows.".format(attr, len(updated)))
+        logger.log(ALERT, f"Set {attr} on  {len(updated)} flows.")
 
     @command.command("flow.decode")
-    def decode(self, flows: typing.Sequence[flow.Flow], part: str) -> None:
+    def decode(self, flows: Sequence[flow.Flow], part: str) -> None:
         """
-            Decode flows.
+        Decode flows.
         """
         updated = []
         for f in flows:
@@ -214,12 +195,12 @@ class Core:
                 p.decode()
                 updated.append(f)
         ctx.master.addons.trigger(hooks.UpdateHook(updated))
-        ctx.log.alert("Decoded %s flows." % len(updated))
+        logger.log(ALERT, "Decoded %s flows." % len(updated))
 
     @command.command("flow.encode.toggle")
-    def encode_toggle(self, flows: typing.Sequence[flow.Flow], part: str) -> None:
+    def encode_toggle(self, flows: Sequence[flow.Flow], part: str) -> None:
         """
-            Toggle flow encoding on and off, using deflate for encoding.
+        Toggle flow encoding on and off, using deflate for encoding.
         """
         updated = []
         for f in flows:
@@ -233,18 +214,18 @@ class Core:
                     p.decode()
                 updated.append(f)
         ctx.master.addons.trigger(hooks.UpdateHook(updated))
-        ctx.log.alert("Toggled encoding on %s flows." % len(updated))
+        logger.log(ALERT, "Toggled encoding on %s flows." % len(updated))
 
     @command.command("flow.encode")
     @command.argument("encoding", type=mitmproxy.types.Choice("flow.encode.options"))
     def encode(
         self,
-        flows: typing.Sequence[flow.Flow],
+        flows: Sequence[flow.Flow],
         part: str,
         encoding: str,
     ) -> None:
         """
-            Encode flows with a specified encoding.
+        Encode flows with a specified encoding.
         """
         updated = []
         for f in flows:
@@ -256,50 +237,46 @@ class Core:
                     p.encode(encoding)
                     updated.append(f)
         ctx.master.addons.trigger(hooks.UpdateHook(updated))
-        ctx.log.alert("Encoded %s flows." % len(updated))
+        logger.log(ALERT, "Encoded %s flows." % len(updated))
 
     @command.command("flow.encode.options")
-    def encode_options(self) -> typing.Sequence[str]:
+    def encode_options(self) -> Sequence[str]:
         """
-            The possible values for an encoding specification.
+        The possible values for an encoding specification.
         """
         return ["gzip", "deflate", "br", "zstd"]
 
     @command.command("options.load")
     def options_load(self, path: mitmproxy.types.Path) -> None:
         """
-            Load options from a file.
+        Load options from a file.
         """
         try:
             optmanager.load_paths(ctx.options, path)
         except (OSError, exceptions.OptionsError) as e:
-            raise exceptions.CommandError(
-                "Could not load options - %s" % e
-            ) from e
+            raise exceptions.CommandError("Could not load options - %s" % e) from e
 
     @command.command("options.save")
     def options_save(self, path: mitmproxy.types.Path) -> None:
         """
-            Save options to a file.
+        Save options to a file.
         """
         try:
             optmanager.save(ctx.options, path)
         except OSError as e:
-            raise exceptions.CommandError(
-                "Could not save options - %s" % e
-            ) from e
+            raise exceptions.CommandError("Could not save options - %s" % e) from e
 
     @command.command("options.reset")
     def options_reset(self) -> None:
         """
-            Reset all options to defaults.
+        Reset all options to defaults.
         """
         ctx.options.reset()
 
     @command.command("options.reset.one")
     def options_reset_one(self, name: str) -> None:
         """
-            Reset one option to its default value.
+        Reset one option to its default value.
         """
         if name not in ctx.options:
             raise exceptions.CommandError("No such option: %s" % name)
