@@ -1,5 +1,7 @@
+import logging
 import shlex
-import typing
+from collections.abc import Callable, Sequence
+from typing import Any, Union
 
 import pyperclip
 
@@ -23,7 +25,7 @@ def cleanup_request(f: flow.Flow) -> http.Request:
 
 def pop_headers(request: http.Request) -> http.Request:
     # Remove some headers that are redundant for curl/httpie export
-    request.headers.pop('content-length')
+    request.headers.pop("content-length")
     if request.headers.get("host", "") == request.host:
         request.headers.pop("host")
     if request.headers.get(":authority", "") == request.host:
@@ -49,10 +51,7 @@ def request_content_for_console(request: http.Request) -> str:
         # see https://github.com/python/cpython/pull/10871
         raise exceptions.CommandError("Request content must be valid unicode")
     escape_control_chars = {chr(i): f"\\x{i:02x}" for i in range(32)}
-    return "".join(
-        escape_control_chars.get(x, x)
-        for x in text
-    )
+    return "".join(escape_control_chars.get(x, x) for x in text)
 
 
 def curl_command(f: flow.Flow) -> str:
@@ -62,8 +61,12 @@ def curl_command(f: flow.Flow) -> str:
 
     server_addr = f.server_conn.peername[0] if f.server_conn.peername else None
 
-    if ctx.options.export_preserve_original_ip and server_addr and request.pretty_host != server_addr:
-        resolve = "{}:{}:[{}]".format(request.pretty_host, request.port, server_addr)
+    if (
+        ctx.options.export_preserve_original_ip
+        and server_addr
+        and request.pretty_host != server_addr
+    ):
+        resolve = f"{request.pretty_host}:{request.port}:[{server_addr}]"
         args.append("--resolve")
         args.append(resolve)
 
@@ -80,7 +83,7 @@ def curl_command(f: flow.Flow) -> str:
 
     if request.content:
         args += ["-d", request_content_for_console(request)]
-    return ' '.join(shlex.quote(arg) for arg in args)
+    return " ".join(shlex.quote(arg) for arg in args)
 
 
 def httpie_command(f: flow.Flow) -> str:
@@ -95,7 +98,7 @@ def httpie_command(f: flow.Flow) -> str:
     args = ["http", request.method, url]
     for k, v in request.headers.items(multi=True):
         args.append(f"{k}: {v}")
-    cmd = ' '.join(shlex.quote(arg) for arg in args)
+    cmd = " ".join(shlex.quote(arg) for arg in args)
     if request.content:
         cmd += " <<< " + shlex.quote(request_content_for_console(request))
     return cmd
@@ -117,8 +120,14 @@ def raw_response(f: flow.Flow) -> bytes:
 
 def raw(f: flow.Flow, separator=b"\r\n\r\n") -> bytes:
     """Return either the request or response if only one exists, otherwise return both"""
-    request_present = isinstance(f, http.HTTPFlow) and f.request and f.request.raw_content is not None
-    response_present = isinstance(f, http.HTTPFlow) and f.response and f.response.raw_content is not None
+    request_present = (
+        isinstance(f, http.HTTPFlow) and f.request and f.request.raw_content is not None
+    )
+    response_present = (
+        isinstance(f, http.HTTPFlow)
+        and f.response
+        and f.response.raw_content is not None
+    )
 
     if request_present and response_present:
         return b"".join([raw_request(f), separator, raw_response(f)])
@@ -130,7 +139,7 @@ def raw(f: flow.Flow, separator=b"\r\n\r\n") -> bytes:
         raise exceptions.CommandError("Can't export flow with no request or response.")
 
 
-formats: typing.Dict[str, typing.Callable[[flow.Flow], typing.Union[str, bytes]]] = dict(
+formats: dict[str, Callable[[flow.Flow], Union[str, bytes]]] = dict(
     curl=curl_command,
     httpie=httpie_command,
     raw=raw,
@@ -142,31 +151,33 @@ formats: typing.Dict[str, typing.Callable[[flow.Flow], typing.Union[str, bytes]]
 class Export:
     def load(self, loader):
         loader.add_option(
-            "export_preserve_original_ip", bool, False,
+            "export_preserve_original_ip",
+            bool,
+            False,
             """
             When exporting a request as an external command, make an effort to
             connect to the same IP as in the original request. This helps with
             reproducibility in cases where the behaviour depends on the
             particular host we are connecting to. Currently this only affects
             curl exports.
-            """
+            """,
         )
 
     @command.command("export.formats")
-    def formats(self) -> typing.Sequence[str]:
+    def formats(self) -> Sequence[str]:
         """
-            Return a list of the supported export formats.
+        Return a list of the supported export formats.
         """
         return list(sorted(formats.keys()))
 
     @command.command("export.file")
     def file(self, format: str, flow: flow.Flow, path: mitmproxy.types.Path) -> None:
         """
-            Export a flow to path.
+        Export a flow to path.
         """
         if format not in formats:
             raise exceptions.CommandError("No such export format: %s" % format)
-        func: typing.Any = formats[format]
+        func: Any = formats[format]
         v = func(flow)
         try:
             with open(path, "wb") as fp:
@@ -175,19 +186,25 @@ class Export:
                 else:
                     fp.write(v.encode("utf-8"))
         except OSError as e:
-            ctx.log.error(str(e))
+            logging.error(str(e))
 
     @command.command("export.clip")
     def clip(self, format: str, f: flow.Flow) -> None:
         """
-            Export a flow to the system clipboard.
+        Export a flow to the system clipboard.
+        """
+        try:
+            pyperclip.copy(self.export_str(format, f))
+        except pyperclip.PyperclipException as e:
+            logging.error(str(e))
+
+    @command.command("export")
+    def export_str(self, format: str, f: flow.Flow) -> str:
+        """
+        Export a flow and return the result.
         """
         if format not in formats:
             raise exceptions.CommandError("No such export format: %s" % format)
         func = formats[format]
 
-        val = strutils.always_str(func(f), "utf8", "backslashreplace")
-        try:
-            pyperclip.copy(val)
-        except pyperclip.PyperclipException as e:
-            ctx.log.error(str(e))
+        return strutils.always_str(func(f), "utf8", "backslashreplace")
