@@ -1,7 +1,10 @@
 import collections
 import enum
+from logging import DEBUG, WARNING
+
 import time
 from dataclasses import dataclass
+from functools import cached_property
 from typing import Optional, Union
 
 import wsproto.handshake
@@ -42,6 +45,7 @@ from ._hooks import (  # noqa
 from ._http1 import Http1Client, Http1Connection, Http1Server
 from ._http2 import Http2Client, Http2Server
 from ...context import Context
+from ...mode_specs import ReverseMode, UpstreamMode
 
 
 class HTTPMode(enum.Enum):
@@ -124,7 +128,7 @@ class HttpStream(layer.Layer):
     stream_id: StreamId
     child_layer: Optional[layer.Layer] = None
 
-    @property
+    @cached_property
     def mode(self):
         i = self.context.layers.index(self)
         parent: HttpLayer = self.context.layers[i - 1]
@@ -222,7 +226,7 @@ class HttpStream(layer.Layer):
 
         # update host header in reverse proxy mode
         if (
-            self.context.options.mode.startswith("reverse:")
+            isinstance(self.context.client.proxy_mode, ReverseMode)
             and not self.context.options.keep_host_header
         ):
             assert self.context.server.address
@@ -260,6 +264,7 @@ class HttpStream(layer.Layer):
             )
         ok = yield from self.make_server_connection()
         if not ok:
+            self.client_state = self.state_errored
             return
         yield SendHttp(
             RequestHeaders(self.stream_id, self.flow.request, end_stream=False),
@@ -489,14 +494,14 @@ class HttpStream(layer.Layer):
             else:
                 yield commands.Log(
                     f"Sent HTTP 101 response, but no protocol is enabled to upgrade to.",
-                    "warn",
+                    WARNING,
                 )
                 yield commands.CloseConnection(self.context.client)
                 self.client_state = self.server_state = self.state_errored
                 return
             if self.debug:
                 yield commands.Log(
-                    f"{self.debug}[http] upgrading to {self.child_layer}", "debug"
+                    f"{self.debug}[http] upgrading to {self.child_layer}", DEBUG
                 )
             yield from self.child_layer.handle_event(events.Start())
             self._handle_event = self.passthrough
@@ -628,8 +633,7 @@ class HttpStream(layer.Layer):
             and self.server_state not in (self.state_done, self.state_errored)
         )
         need_error_hook = not (
-            self.client_state
-            in (self.state_wait_for_request_headers, self.state_errored)
+            self.client_state == self.state_errored
             or self.server_state in (self.state_done, self.state_errored)
         )
 
@@ -690,7 +694,9 @@ class HttpStream(layer.Layer):
             if err:
                 self.flow.response = http.Response.make(
                     502,
-                    f"Cannot connect to {human.format_address(self.context.server.address)}: {err}",
+                    f"Cannot connect to {human.format_address(self.context.server.address)}: {err} "
+                    f"If you plan to redirect requests away from this server, "
+                    f"consider setting `connection_strategy` to `lazy` to suppress early connections."
                 )
         self.child_layer = layer.NextLayer(self.context)
         yield from self.handle_connect_finish()
@@ -835,9 +841,9 @@ class HttpLayer(layer.Layer):
         if isinstance(event, events.Start):
             yield from self.event_to_child(self.connections[self.context.client], event)
             if self.mode is HTTPMode.upstream:
-                self.context.server.via = server_spec.parse_with_mode(
-                    self.context.options.mode
-                )[1]
+                proxy_mode = self.context.client.proxy_mode
+                assert isinstance(proxy_mode, UpstreamMode)
+                self.context.server.via = (proxy_mode.scheme, proxy_mode.address)
         elif isinstance(event, events.Wakeup):
             stream = self.command_sources.pop(event.command)
             yield from self.event_to_child(stream, event)
@@ -910,10 +916,9 @@ class HttpLayer(layer.Layer):
                 try:
                     stream = self.streams[command.event.stream_id]
                 except KeyError:
-                    # We may be getting errors for a specific stream even though we've already finished handling it.
-                    assert isinstance(
-                        command.event, (RequestProtocolError, ResponseProtocolError)
-                    )
+                    # We may be getting data or errors for a stream even though we've already finished handling it,
+                    # see for example https://github.com/mitmproxy/mitmproxy/issues/5343.
+                    pass
                 else:
                     yield from self.event_to_child(stream, command.event)
             elif isinstance(command, SendHttp):
@@ -997,7 +1002,6 @@ class HttpLayer(layer.Layer):
 
             if event.via:
                 context.server.via = event.via
-                assert event.via.scheme in ("http", "https")
                 # We always send a CONNECT request, *except* for plaintext absolute-form HTTP requests in upstream mode.
                 send_connect = event.tls or self.mode != HTTPMode.upstream
                 stack /= _upstream_proxy.HttpUpstreamProxy.make(context, send_connect)
