@@ -1,12 +1,14 @@
+from functools import lru_cache
+from typing import Optional
+
 import urwid
 
+import mitmproxy.tools.console.master
 from mitmproxy.tools.console import common
 from mitmproxy.tools.console import layoutwidget
-import mitmproxy.tools.console.master # noqa
 
 
 class FlowItem(urwid.WidgetWrap):
-
     def __init__(self, master, flow):
         self.master, self.flow = master, flow
         w = self.get_text()
@@ -14,12 +16,17 @@ class FlowItem(urwid.WidgetWrap):
 
     def get_text(self):
         cols, _ = self.master.ui.get_cols_rows()
+        layout = self.master.options.console_flowlist_layout
+        if layout == "list" or (layout == "default" and cols < 100):
+            render_mode = common.RenderMode.LIST
+        else:
+            render_mode = common.RenderMode.TABLE
+
         return common.format_flow(
             self.flow,
-            self.flow is self.master.view.focus.flow,
+            render_mode=render_mode,
+            focused=self.flow is self.master.view.focus.flow,
             hostheader=self.master.options.showhost,
-            cols=cols,
-            layout=self.master.options.console_flowlist_layout
         )
 
     def selectable(self):
@@ -27,15 +34,15 @@ class FlowItem(urwid.WidgetWrap):
 
     def mouse_event(self, size, event, button, col, row, focus):
         if event == "mouse press" and button == 1:
-            if self.flow.request:
-                self.master.commands.execute("console.view.flow @focus")
-                return True
+            self.master.commands.execute("console.view.flow @focus")
+            return True
 
     def keypress(self, size, key):
         return key
 
 
 class FlowListWalker(urwid.ListWalker):
+    master: "mitmproxy.tools.console.master.ConsoleMaster"
 
     def __init__(self, master):
         self.master = master
@@ -43,13 +50,14 @@ class FlowListWalker(urwid.ListWalker):
     def positions(self, reverse=False):
         # The stub implementation of positions can go once this issue is resolved:
         # https://github.com/urwid/urwid/issues/294
-        ret = range(self.master.commands.execute("view.properties.length"))
+        ret = range(self.master.view.get_length())
         if reverse:
             return reversed(ret)
         return ret
 
     def view_changed(self):
         self._modified()
+        self._get.cache_clear()
 
     def get_focus(self):
         if not self.master.view.focus.flow:
@@ -61,33 +69,28 @@ class FlowListWalker(urwid.ListWalker):
         if self.master.commands.execute("view.properties.inbounds %d" % index):
             self.master.view.focus.index = index
 
-    def get_next(self, pos):
-        pos = pos + 1
-        if not self.master.commands.execute("view.properties.inbounds %d" % pos):
+    @lru_cache(maxsize=None)
+    def _get(self, pos: int) -> tuple[Optional[FlowItem], Optional[int]]:
+        if not self.master.view.inbounds(pos):
             return None, None
-        f = FlowItem(self.master, self.master.view[pos])
-        return f, pos
+        return FlowItem(self.master, self.master.view[pos]), pos
+
+    def get_next(self, pos):
+        return self._get(pos + 1)
 
     def get_prev(self, pos):
-        pos = pos - 1
-        if not self.master.commands.execute("view.properties.inbounds %d" % pos):
-            return None, None
-        f = FlowItem(self.master, self.master.view[pos])
-        return f, pos
+        return self._get(pos - 1)
 
 
 class FlowListBox(urwid.ListBox, layoutwidget.LayoutWidget):
     title = "Flows"
     keyctx = "flowlist"
 
-    def __init__(
-        self, master: "mitmproxy.tools.console.master.ConsoleMaster"
-    ) -> None:
+    def __init__(self, master: "mitmproxy.tools.console.master.ConsoleMaster") -> None:
         self.master: "mitmproxy.tools.console.master.ConsoleMaster" = master
         super().__init__(FlowListWalker(master))
         self.master.options.subscribe(
-            self.set_flowlist_layout,
-            ["console_flowlist_layout"]
+            self.set_flowlist_layout, ["console_flowlist_layout"]
         )
 
     def keypress(self, size, key):
@@ -102,5 +105,5 @@ class FlowListBox(urwid.ListBox, layoutwidget.LayoutWidget):
     def view_changed(self):
         self.body.view_changed()
 
-    def set_flowlist_layout(self, opts, updated):
+    def set_flowlist_layout(self, *_) -> None:
         self.master.ui.clear()
