@@ -1,22 +1,24 @@
 import abc
 import copy
 import os
-import typing
+from collections.abc import Callable, Container, Iterable, Sequence
+from typing import Any, AnyStr, Optional
+
 import urwid
 
 from mitmproxy.utils import strutils
 from mitmproxy import exceptions
 from mitmproxy.tools.console import signals
 from mitmproxy.tools.console import layoutwidget
-import mitmproxy.tools.console.master # noqa
+import mitmproxy.tools.console.master
 
 
-def read_file(filename: str, escaped: bool) -> typing.AnyStr:
+def read_file(filename: str, escaped: bool) -> AnyStr:
     filename = os.path.expanduser(filename)
     try:
         with open(filename, "r" if escaped else "rb") as f:
             d = f.read()
-    except IOError as v:
+    except OSError as v:
         raise exceptions.CommandError(v)
     if escaped:
         try:
@@ -53,28 +55,27 @@ class Column(metaclass=abc.ABCMeta):
         pass
 
     @abc.abstractmethod
-    def blank(self) -> typing.Any:
+    def blank(self) -> Any:
         pass
 
-    def keypress(self, key: str, editor: "GridEditor") -> typing.Optional[str]:
+    def keypress(self, key: str, editor: "GridEditor") -> Optional[str]:
         return key
 
 
 class GridRow(urwid.WidgetWrap):
-
     def __init__(
-            self,
-            focused: typing.Optional[int],
-            editing: bool,
-            editor: "GridEditor",
-            values: typing.Tuple[typing.Iterable[bytes], typing.Container[int]]
+        self,
+        focused: Optional[int],
+        editing: bool,
+        editor: "GridEditor",
+        values: tuple[Iterable[bytes], Container[int]],
     ) -> None:
         self.focused = focused
         self.editor = editor
-        self.edit_col: typing.Optional[Cell] = None
+        self.edit_col: Optional[Cell] = None
 
         errors = values[1]
-        self.fields: typing.Sequence[typing.Any] = []
+        self.fields: Sequence[Any] = []
         for i, v in enumerate(values[0]):
             if focused == i and editing:
                 self.edit_col = self.editor.columns[i].Edit(v)
@@ -93,10 +94,7 @@ class GridRow(urwid.WidgetWrap):
         fspecs = self.fields[:]
         if len(self.fields) > 1:
             fspecs[0] = ("fixed", self.editor.first_width + 2, fspecs[0])
-        w = urwid.Columns(
-            fspecs,
-            dividechars=2
-        )
+        w = urwid.Columns(fspecs, dividechars=2)
         if focused is not None:
             w.set_focus_column(focused)
         super().__init__(w)
@@ -113,41 +111,35 @@ class GridRow(urwid.WidgetWrap):
 
 class GridWalker(urwid.ListWalker):
     """
-        Stores rows as a list of (rows, errors) tuples, where rows is a list
-        and errors is a set with an entry of each offset in rows that is an
-        error.
+    Stores rows as a list of (rows, errors) tuples, where rows is a list
+    and errors is a set with an entry of each offset in rows that is an
+    error.
     """
 
-    def __init__(
-            self,
-            lst: typing.Iterable[list],
-            editor: "GridEditor"
-    ) -> None:
-        self.lst: typing.Sequence[typing.Tuple[typing.Any, typing.Set]] = [(i, set()) for i in lst]
+    def __init__(self, lst: Iterable[list], editor: "GridEditor") -> None:
+        self.lst: Sequence[tuple[Any, set]] = [(i, set()) for i in lst]
         self.editor = editor
         self.focus = 0
         self.focus_col = 0
-        self.edit_row: typing.Optional[GridRow] = None
+        self.edit_row: Optional[GridRow] = None
 
     def _modified(self):
         self.editor.show_empty_msg()
         return super()._modified()
 
     def add_value(self, lst):
-        self.lst.append(
-            (lst[:], set())
-        )
+        self.lst.append((lst[:], set()))
         self._modified()
 
     def get_current_value(self):
         if self.lst:
             return self.lst[self.focus][0][self.focus_col]
 
-    def set_current_value(self, val):
+    def set_current_value(self, val) -> None:
         errors = self.lst[self.focus][1]
         emsg = self.editor.is_error(self.focus_col, val)
         if emsg:
-            signals.status_message.send(message=emsg, expire=5)
+            signals.status_message.send(message=emsg)
             errors.add(self.focus_col)
         else:
             errors.discard(self.focus_col)
@@ -155,7 +147,7 @@ class GridWalker(urwid.ListWalker):
 
     def set_value(self, val, focus, focus_col, errors=None):
         if not errors:
-            errors = set([])
+            errors = set()
         row = list(self.lst[focus][0])
         row[focus_col] = val
         self.lst[focus] = [tuple(row), errors]
@@ -169,10 +161,7 @@ class GridWalker(urwid.ListWalker):
 
     def _insert(self, pos):
         self.focus = pos
-        self.lst.insert(
-            self.focus,
-            ([c.blank() for c in self.editor.columns], set([]))
-        )
+        self.lst.insert(self.focus, ([c.blank() for c in self.editor.columns], set()))
         self.focus_col = 0
         self.start_edit()
 
@@ -220,12 +209,10 @@ class GridWalker(urwid.ListWalker):
         if self.edit_row:
             return self.edit_row, self.focus
         elif self.lst:
-            return GridRow(
-                self.focus_col,
-                False,
-                self.editor,
-                self.lst[self.focus]
-            ), self.focus
+            return (
+                GridRow(self.focus_col, False, self.editor, self.lst[self.focus]),
+                self.focus,
+            )
         else:
             return None, None
 
@@ -258,14 +245,14 @@ class BaseGridEditor(urwid.WidgetWrap):
     keyctx = "grideditor"
 
     def __init__(
-            self,
-            master: "mitmproxy.tools.console.master.ConsoleMaster",
-            title,
-            columns,
-            value: typing.Any,
-            callback: typing.Callable[..., None],
-            *cb_args,
-            **cb_kwargs
+        self,
+        master: "mitmproxy.tools.console.master.ConsoleMaster",
+        title,
+        columns,
+        value: Any,
+        callback: Callable[..., None],
+        *cb_args,
+        **cb_kwargs
     ) -> None:
         value = self.data_in(copy.deepcopy(value))
         self.master = master
@@ -292,10 +279,7 @@ class BaseGridEditor(urwid.WidgetWrap):
                     headings.append(("fixed", first_width + 2, c))
                 else:
                     headings.append(c)
-            h = urwid.Columns(
-                headings,
-                dividechars=2
-            )
+            h = urwid.Columns(headings, dividechars=2)
             h = urwid.AttrWrap(h, "heading")
 
         self.walker = GridWalker(self.value, self)
@@ -356,22 +340,22 @@ class BaseGridEditor(urwid.WidgetWrap):
         elif column.keypress(key, self) and not self.handle_key(key):
             return self._w.keypress(size, key)
 
-    def data_out(self, data: typing.Sequence[list]) -> typing.Any:
+    def data_out(self, data: Sequence[list]) -> Any:
         """
-            Called on raw list data, before data is returned through the
-            callback.
+        Called on raw list data, before data is returned through the
+        callback.
         """
         return data
 
-    def data_in(self, data: typing.Any) -> typing.Iterable[list]:
+    def data_in(self, data: Any) -> Iterable[list]:
         """
-            Called to prepare provided data.
+        Called to prepare provided data.
         """
         return data
 
-    def is_error(self, col: int, val: typing.Any) -> typing.Optional[str]:
+    def is_error(self, col: int, val: Any) -> Optional[str]:
         """
-            Return None, or a string error message.
+        Return None, or a string error message.
         """
         return None
 
@@ -403,32 +387,27 @@ class BaseGridEditor(urwid.WidgetWrap):
 
 class GridEditor(BaseGridEditor):
     title = ""
-    columns: typing.Sequence[Column] = ()
+    columns: Sequence[Column] = ()
     keyctx = "grideditor"
 
     def __init__(
-            self,
-            master: "mitmproxy.tools.console.master.ConsoleMaster",
-            value: typing.Any,
-            callback: typing.Callable[..., None],
-            *cb_args,
-            **cb_kwargs
+        self,
+        master: "mitmproxy.tools.console.master.ConsoleMaster",
+        value: Any,
+        callback: Callable[..., None],
+        *cb_args,
+        **cb_kwargs
     ) -> None:
         super().__init__(
-            master,
-            self.title,
-            self.columns,
-            value,
-            callback,
-            *cb_args,
-            **cb_kwargs
+            master, self.title, self.columns, value, callback, *cb_args, **cb_kwargs
         )
 
 
 class FocusEditor(urwid.WidgetWrap, layoutwidget.LayoutWidget):
     """
-        A specialised GridEditor that edits the current focused flow.
+    A specialised GridEditor that edits the current focused flow.
     """
+
     keyctx = "grideditor"
 
     def __init__(self, master):
@@ -441,19 +420,19 @@ class FocusEditor(urwid.WidgetWrap, layoutwidget.LayoutWidget):
 
     def get_data(self, flow):
         """
-            Retrieve the data to edit from the current flow.
+        Retrieve the data to edit from the current flow.
         """
         raise NotImplementedError
 
     def set_data(self, vals, flow):
         """
-            Set the current data on the flow.
+        Set the current data on the flow.
         """
         raise NotImplementedError
 
-    def set_data_update(self, vals, flow):
+    def set_data_update(self, vals, flow) -> None:
         self.set_data(vals, flow)
-        signals.flow_change.send(self, flow = flow)
+        signals.flow_change.send(flow=flow)
 
     def key_responder(self):
         return self._w
