@@ -34,8 +34,8 @@ class Display(base.Cell):
 
 class Edit(base.Cell):
     def __init__(self, data: bytes) -> None:
-        data = strutils.bytes_to_escaped_str(data)
-        w = urwid.Edit(edit_text=data, wrap="any", multiline=True)
+        d = strutils.bytes_to_escaped_str(data)
+        w = urwid.Edit(edit_text=d, wrap="any", multiline=True)
         w = urwid.AttrWrap(w, "editfield")
         super().__init__(w)
 
@@ -44,9 +44,5 @@ class Edit(base.Cell):
         try:
             return strutils.escaped_str_to_bytes(txt)
         except ValueError:
-            signals.status_message.send(
-                self,
-                message="Invalid data.",
-                expire=1000
-            )
+            signals.status_message.send(message="Invalid data.")
             raise
