@@ -18,12 +18,10 @@ class Column(base.Column):
     def blank(self):
         return []
 
-    def keypress(self, key, editor):
+    def keypress(self, key: str, editor):
         if key in "rRe":
             signals.status_message.send(
-                self,
-                message="Press enter to edit this field.",
-                expire=1000
+                message="Press enter to edit this field."
             )
             return
         elif key == "m_select":
