@@ -1,5 +1,8 @@
-import typing
+import logging
 import os
+from collections.abc import Sequence
+from functools import cache
+from typing import Optional
 
 import ruamel.yaml
 import ruamel.yaml.error
@@ -33,9 +36,16 @@ Contexts = {
 
 
 navkeys = [
-    "m_start", "m_end", "m_next", "m_select",
-    "up", "down", "page_up", "page_down",
-    "left", "right"
+    "m_start",
+    "m_end",
+    "m_next",
+    "m_select",
+    "up",
+    "down",
+    "page_up",
+    "page_down",
+    "left",
+    "right",
 ]
 
 
@@ -46,11 +56,14 @@ class Binding:
 
     def keyspec(self):
         """
-            Translate the key spec from a convenient user specification to one
-            Urwid understands.
+        Translate the key spec from a convenient user specification to one
+        Urwid understands.
         """
         return self.key.replace("space", " ")
 
+    def key_short(self) -> str:
+        return self.key.replace("enter", "⏎").replace("right", "→").replace("space", "␣")
+
     def sortkey(self):
         return self.key + ",".join(self.contexts)
 
@@ -70,15 +83,13 @@ class Keymap:
             if c not in Contexts:
                 raise ValueError("Unsupported context: %s" % c)
 
-    def add(
-        self,
-        key: str,
-        command: str,
-        contexts: typing.Sequence[str],
-        help=""
-    ) -> None:
+    def _on_change(self) -> None:
+        signals.keybindings_change.send()
+        self.binding_for_help.cache_clear()
+
+    def add(self, key: str, command: str, contexts: Sequence[str], help="") -> None:
         """
-            Add a key to the key map.
+        Add a key to the key map.
         """
         self._check_contexts(contexts)
 
@@ -94,11 +105,11 @@ class Keymap:
             b = Binding(key=key, command=command, contexts=contexts, help=help)
             self.bindings.append(b)
             self.bind(b)
-        signals.keybindings_change.send(self)
+        self._on_change()
 
-    def remove(self, key: str, contexts: typing.Sequence[str]) -> None:
+    def remove(self, key: str, contexts: Sequence[str]) -> None:
         """
-            Remove a key from the key map.
+        Remove a key from the key map.
         """
         self._check_contexts(contexts)
         for c in contexts:
@@ -109,7 +120,7 @@ class Keymap:
                 if b.contexts:
                     self.bindings.append(b)
                     self.bind(b)
-        signals.keybindings_change.send(self)
+        self._on_change()
 
     def bind(self, binding: Binding) -> None:
         for c in binding.contexts:
@@ -117,18 +128,26 @@ class Keymap:
 
     def unbind(self, binding: Binding) -> None:
         """
-            Unbind also removes the binding from the list.
+        Unbind also removes the binding from the list.
         """
         for c in binding.contexts:
             del self.keys[c][binding.keyspec()]
             self.bindings = [b for b in self.bindings if b != binding]
+        self._on_change()
 
-    def get(self, context: str, key: str) -> typing.Optional[Binding]:
+    def get(self, context: str, key: str) -> Optional[Binding]:
         if context in self.keys:
             return self.keys[context].get(key, None)
         return None
 
-    def list(self, context: str) -> typing.Sequence[Binding]:
+    @cache
+    def binding_for_help(self, help: str) -> Optional[Binding]:
+        for b in self.bindings:
+            if b.help == help:
+                return b
+        return None
+
+    def list(self, context: str) -> Sequence[Binding]:
         b = [x for x in self.bindings if context in x.contexts or context == "all"]
         single = [x for x in b if len(x.key.split()) == 1]
         multi = [x for x in b if len(x.key.split()) != 1]
@@ -136,19 +155,19 @@ class Keymap:
         multi.sort(key=lambda x: x.sortkey())
         return single + multi
 
-    def handle(self, context: str, key: str) -> typing.Optional[str]:
+    def handle(self, context: str, key: str) -> Optional[str]:
         """
-            Returns the key if it has not been handled, or None.
+        Returns the key if it has not been handled, or None.
         """
         b = self.get(context, key) or self.get("global", key)
         if b:
             return self.executor(b.command)
         return key
 
-    def handle_only(self, context: str, key: str) -> typing.Optional[str]:
+    def handle_only(self, context: str, key: str) -> Optional[str]:
         """
-            Like handle, but ignores global bindings. Returns the key if it has
-            not been handled, or None.
+        Like handle, but ignores global bindings. Returns the key if it has
+        not been handled, or None.
         """
         b = self.get(context, key)
         if b:
@@ -173,9 +192,7 @@ class KeymapConfig:
         try:
             self.load_path(ctx.master.keymap, path)  # type: ignore
         except (OSError, KeyBindingError) as e:
-            raise exceptions.CommandError(
-                "Could not load key bindings - %s" % e
-            ) from e
+            raise exceptions.CommandError("Could not load key bindings - %s" % e) from e
 
     def running(self):
         p = os.path.join(os.path.expanduser(ctx.options.confdir), self.defaultFile)
@@ -183,7 +200,7 @@ class KeymapConfig:
             try:
                 self.load_path(ctx.master.keymap, p)
             except KeyBindingError as e:
-                ctx.log.error(e)
+                logging.error(e)
 
     def load_path(self, km, p):
         if os.path.exists(p) and os.path.isfile(p):
@@ -191,40 +208,34 @@ class KeymapConfig:
                 try:
                     txt = f.read()
                 except UnicodeDecodeError as e:
-                    raise KeyBindingError(
-                        f"Encoding error - expected UTF8: {p}: {e}"
-                    )
+                    raise KeyBindingError(f"Encoding error - expected UTF8: {p}: {e}")
             try:
                 vals = self.parse(txt)
             except KeyBindingError as e:
-                raise KeyBindingError(
-                    f"Error reading {p}: {e}"
-                ) from e
+                raise KeyBindingError(f"Error reading {p}: {e}") from e
             for v in vals:
                 user_ctxs = v.get("ctx", ["global"])
                 try:
                     km._check_contexts(user_ctxs)
                     km.remove(v["key"], user_ctxs)
                     km.add(
-                        key = v["key"],
-                        command = v["cmd"],
-                        contexts = user_ctxs,
-                        help = v.get("help", None),
+                        key=v["key"],
+                        command=v["cmd"],
+                        contexts=user_ctxs,
+                        help=v.get("help", None),
                     )
                 except ValueError as e:
-                    raise KeyBindingError(
-                        f"Error reading {p}: {e}"
-                    ) from e
+                    raise KeyBindingError(f"Error reading {p}: {e}") from e
 
     def parse(self, text):
         try:
-            data = ruamel.yaml.YAML(typ='safe', pure=True).load(text)
+            data = ruamel.yaml.YAML(typ="safe", pure=True).load(text)
         except ruamel.yaml.error.MarkedYAMLError as v:
             if hasattr(v, "problem_mark"):
                 snip = v.problem_mark.get_snippet()
                 raise KeyBindingError(
-                    "Key binding config error at line %s:\n%s\n%s" %
-                    (v.problem_mark.line + 1, snip, v.problem)
+                    "Key binding config error at line %s:\n%s\n%s"
+                    % (v.problem_mark.line + 1, snip, v.problem)
                 )
             else:
                 raise KeyBindingError("Could not parse key bindings.")
